package de.tu_darmstadt.tk.SmartHomeNetworkSim.core.scheduler;

import java.util.TreeSet;

/**
 * The Scheduler manages the EventQueue of the framework and allows manipulating the queue.
 *
 * @author Andreas T. Meyer-Berg
 */
public class Scheduler {

	/**
	 * Event Queue, which stores the different events
	 */
	private TreeSet<Schedulable> eventQueue;
	
	/**
	 * Minimum time step. No earlier events can be scheduled
	 */
	private long minimumTimeStep = Long.MIN_VALUE;
	
	/**
	 * Initializes a new Scheduler with an empty event queue.
	 */
	public Scheduler() {
		eventQueue  = new TreeSet<Schedulable>(new ScheduleComparator());
	}
	
	
	/**
	 * Schedules the given {@link Schedulable}, by adding it to the event queue. 
	 * @param event Event which should be scheduled
	 * @return true if it was scheduled, false if otherwise
	 */
	public boolean scheduleEvent(Schedulable event){
		return eventQueue.add(event);
	}
	
	/**
	 * Resets the EventQueue by removing all events and resetting the minimum timestep
	 */
	public void reset(long newMinTime){
		minimumTimeStep = newMinTime;
		eventQueue.clear();
	}
	
	/**
	 * Returns the minimum Time, events could be scheduled.
	 * @return minimum time step
	 */
	public long getMinTime(){
		return minimumTimeStep;
	}
	
	/**
	 * Sets the new minimum time. No events will be scheduled earlier.
	 * @param newMinTime new minimum time
	 */
	public void setMinTime(long newMinTime){
		this.minimumTimeStep = newMinTime;
	}
}
